--[[
        Engine Of Love - a framework for the LÖVE game engine
        Copyright (C) 2013 vitaminx (vitaminx@callistix.net)

        This program is free software: you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation, either version 3 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program.  If not, see <http://www.gnu.org/licenses/>.
--]]

eol.screen={}

function eol.screen.init()
    -- set default line style
    love.graphics.setLineStyle("rough")

    -- get and sort graphics modes
    eol.screen.modes=love.graphics.getModes()
    table.sort(eol.screen.modes,function(a,b) return a.width*a.height<b.width*b.height end)

    -- if requested screen mode not supported, fall back to first mode in list
    if not love.graphics.checkMode(prefs.screen.width.v,prefs.screen.height.v,prefs.screen.fullscreen.v) then
        prefs.screen.width.v=eol.screen.modes[1].width
        prefs.screen.height.v=eol.screen.modes[1].height
    end

    -- set screen mode and mouse pointer
    love.graphics.setMode(prefs.screen.width.v,prefs.screen.height.v,prefs.screen.fullscreen.v,prefs.screen.vsync.v,prefs.screen.fsaa.v)
    if not prefs.screen.pointer.v then love.mouse.setVisible(false) end

    -- create canvas if supported
    if love.graphics.isSupported("canvas") and prefs.screen.usecanvas.v then
        eol.screen.canvas_sup=true
        eol.screen.canvas=love.graphics.newCanvas(prefs.screen.canvas_w.v,prefs.screen.canvas_h.v)
        eol.screen.canvas:setFilter("nearest","nearest")
    else eol.screen.canvas_sup=false end

    -- calculate scaling and position of drawing area
    eol.screen.calcscale(prefs.screen.width.v,prefs.screen.height.v)

    -- get initial timestamp if fps is capped
    if prefs.screen.fpslimit.v>0 then eol.screen.dtnext=love.timer.getMicroTime() end
end

function eol.screen.update(_dt)
    -- add to timer if fps is capped
    if prefs.screen.fpslimit.v>0 then eol.screen.dtnext=eol.screen.dtnext+1/prefs.screen.fpslimit.v end
end

function eol.screen.drawpre()
    -- switch to canvas if supported
    if eol.screen.canvas_sup then love.graphics.setCanvas(eol.screen.canvas) end

    -- clean background
    love.graphics.setBackgroundColor(0,0,0,255)
    love.graphics.clear()
end

function eol.screen.drawpost()
    if eol.screen.canvas_sup then
        -- switch to screen and clean it
        love.graphics.setCanvas()
        love.graphics.setBlendMode("premultiplied")
        love.graphics.setBackgroundColor(0,0,0,255)
        love.graphics.setColor(255,255,255,255)
        love.graphics.clear()

        -- blit scaled and translated canvas to screen
        love.graphics.draw(eol.screen.canvas,eol.screen.ofs_x,eol.screen.ofs_y,0,eol.screen.scale)
        love.graphics.setBlendMode("alpha")
    end
    
    -- sleep if fps is capped
    if prefs.screen.fpslimit.v>0 then
        local _time=love.timer.getMicroTime()
        if eol.screen.dtnext<=_time then
            eol.screen.dtnext=_time
            return
        end
        love.timer.sleep(eol.screen.dtnext-_time)
    end
end

function eol.screen.keydown(_key)
    for _k,_v in pairs(keys.screen) do

        -- switch to smaller screen mode
        if _k=="smaller" and _v.v==_key then
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode() 
            for _i,_v in ipairs(eol.screen.modes) do
                if _v.width==_w and _v.height==_h then
                    local _index=_i-1
                    if _index<1 then _index=#eol.screen.modes end
                    love.graphics.setMode(eol.screen.modes[_index].width,eol.screen.modes[_index].height,_fs,_vs,_aa)
                    eol.screen.calcscale(eol.screen.modes[_index].width,eol.screen.modes[_index].height)
                    eol.osd.msg=eol.screen.modes[_index].width.."x"..eol.screen.modes[_index].height
                    break
                end
            end
            break end

        -- switch to bigger screen mode
        if _k=="bigger" and _v.v==_key then
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode() 
            for _i,_v in ipairs(eol.screen.modes) do
                if _v.width==_w and _v.height==_h then
                    local _index=_i+1
                    if _index>#eol.screen.modes then _index=1 end
                    love.graphics.setMode(eol.screen.modes[_index].width,eol.screen.modes[_index].height,_fs,_vs,_aa)
                    eol.screen.calcscale(eol.screen.modes[_index].width,eol.screen.modes[_index].height)
                    eol.osd.msg=eol.screen.modes[_index].width.."x"..eol.screen.modes[_index].height
                    break
                end
            end
            break end

        -- toggle fullscreen
        if _k=="fullscreen" and _v.v==_key then
            love.graphics.toggleFullscreen()
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode()
            if _fs then eol.osd.msg="FULLSCREEN\nON" else eol.osd.msg="FULLSCREEN\nOFF" end
            break end

        -- toggle vsync
        if _k=="vsync" and _v.v==_key then
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode() 
            if _vs then love.graphics.setMode(_w,_h,_fs,false,_aa) else love.graphics.setMode(_w,_h,_fs,true,_aa) end
            _w,_h,_fs,_vs,_aa=love.graphics.getMode()
            if _vs then eol.osd.msg="VSYNC\nON" else eol.osd.msg="VSYNC\nOFF" end
            break end

        -- decrease FSAA samples
        if _k=="fsaa_less" and _v.v==_key then
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode()
            local _old_aa=_aa
            _aa=_aa-1
            if _aa<0 then _aa=0 end
            love.graphics.setMode(_w,_h,_fs,_vs,_aa)
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode()
            if _old_aa~=_aa then eol.osd.msg="FSAA x ".._aa end
            break end

        -- increase FSAA samples
        if _k=="fsaa_more" and _v.v==_key then
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode()
            local _old_aa=_aa
            _aa=_aa+1
            if _aa>128 then _aa=128 end
            love.graphics.setMode(_w,_h,_fs,_vs,_aa)
            local _w,_h,_fs,_vs,_aa=love.graphics.getMode()
            if _old_aa~=_aa then eol.osd.msg="FSAA x ".._aa end
            break end

        -- decrease FPS limit
        if _k=="fps_less" and _v.v==_key then
            prefs.screen.fpslimit.v=math.floor(prefs.screen.fpslimit.v-5)
            if prefs.screen.fpslimit.v<0 then prefs.screen.fpslimit.v=0 end
            break end

        -- increase FPS limit
        if _k=="fps_more" and _v.v==_key then
            prefs.screen.fpslimit.v=math.floor(prefs.screen.fpslimit.v+5)
            break end
    end
end

function eol.screen.calcscale(_w,_h)
    -- calculate aspect ratios
    local _aspect_win=_w/_h
    local _aspect_can=prefs.screen.canvas_w.v/prefs.screen.canvas_h.v

    if _aspect_win<_aspect_can then
        -- we need vertical offset, calculate it
        eol.screen.scale = _w/prefs.screen.canvas_w.v
        eol.screen.ofs_x = 0
        eol.screen.ofs_y = (_h-(prefs.screen.canvas_h.v*eol.screen.scale))/2
    else
        -- we need horizontal offset, calculate it
        eol.screen.scale = _h/prefs.screen.canvas_h.v
        eol.screen.ofs_x = (_w-(prefs.screen.canvas_w.v*eol.screen.scale))/2
        eol.screen.ofs_y = 0
    end

    -- if multiscale is enabled (scale canvas only to multiple sizes), find right scale and offsets
    if prefs.screen.multiscale.v then
        local _intscale=math.floor(eol.screen.scale)
        local _hgap=_w-prefs.screen.canvas_w.v*_intscale
        local _vgap=_h-prefs.screen.canvas_h.v*_intscale

        if _hgap>0 then eol.screen.ofs_x=_hgap/2 end
        if _vgap>0 then eol.screen.ofs_y=_vgap/2 end

        eol.screen.scale=_intscale
    end

    if eol.screen.canvas_sup then
        -- if we have canvas support, set helper variables to zero
        eol.screen.ofs_x_nocan = 0
        eol.screen.ofs_y_nocan = 0
    else
        -- if canvas is unsupported, set helper variables to offset values
        eol.screen.ofs_x_nocan = (_w-prefs.screen.canvas_w.v)/2
        eol.screen.ofs_y_nocan = (_h-prefs.screen.canvas_h.v)/2
    end
end
