-- The following optional block (until love.load()) is a mechanism for easy enabling/disabling EOL from within your code.
-- If the EOL folder can't be found, it will be autodisabled.

-- change this variable to "false" if you want to disable EOL
use_eol=true

-- check if the EOL folder exists inside the game's main directory, if not EOL will be disabled alltogether
if use_eol then use_eol=love.filesystem.exists("engineoflove") end

-- if EOL is enabled and the folder exists, load all the EOL stuff
if use_eol then require "engineoflove" end

function love.load()
    -- The following block is an example on how to use a user defined debug table.
    -- "n" contains the displayed name, "f" contains a function to return the actual value to display.
    -- Remember that the debug information window can be toggled with F11 by default.
    -- F12 switches between our user defined debug information and a predefined system debug page provided by EOL.

    -- some random game variables
    game   = { status="running" }
    player = { x=50, y=40, vel=50 }

    -- our own debug table which will be displayed on top of everything in the canvas
    if use_eol then eol.debug.user = {
        { n="game.status",      f=function() return game.status end },
        { n="os.date",          f=os.date },
        { n="player position",  f=function() local _s="x:"..math.floor(player.x)..", y:"..math.floor(player.y) return _s end },
    } end
end

function love.update(dt)
    -- we pause the game if window looses the focus
    if eol.screen.focus then game.status="running" else game.status="pause" end

    -- only update the game if it's running
    if game.status=="running" then
        -- some code to move the imaginary player
        if love.keyboard.isDown("left")  then player.x=player.x-dt*player.vel end
        if love.keyboard.isDown("right") then player.x=player.x+dt*player.vel end
        if love.keyboard.isDown("up")    then player.y=player.y-dt*player.vel end
        if love.keyboard.isDown("down")  then player.y=player.y+dt*player.vel end
    end
end

function love.draw()
    --  EOL provides offset values to center everything we draw inside the window.
    --  The following code is an example how to add the values to all x and y coordinates even on systems without canvas support.
    --  It should remain at the beginning of love.draw()

    -- we use zero values for offsets in case EOL is disabled
    local _xeol=0
    local _yeol=0
    -- set the canvas with to window width and height in case EOL is disabled
    local _weol=love.graphics.getWidth()
    local _heol=love.graphics.getHeight()

    if use_eol then
        -- if EOL is enabled, we copy the offset values from EOL variables
        _xeol=eol.screen.ofs_x_nocan
        _yeol=eol.screen.ofs_y_nocan
        -- we also get the canvas dimensions from the preferences
        _weol=prefs.screen.canvas_w.v
        _heol=prefs.screen.canvas_h.v
    end

    -- Here you can add your own drawing routines, the following code is an example.
    -- The following code is an example how to use the offset variables and how to get the dimensions of the drawable area

    -- we fill the whole drawable area with a background color
    love.graphics.setColor(26,12,34,255)
    love.graphics.rectangle("fill",0,0,_weol,_heol)

    -- if game status is running, display game graphics
    if game.status=="running" then
        -- from now on we add the offset values to all positions, for example:
        love.graphics.setColor(88,165,116,255)
        love.graphics.rectangle("fill",_xeol+140,_yeol+170,40,40)

        love.graphics.setColor(34,176,56,255)
        love.graphics.rectangle("fill",_xeol+player.x,_yeol+player.y,30,30)
    -- if game is paused, display text
    elseif game.status=="pause" then
        local _text="PAUSE"
        local _font=love.graphics.setNewFont(22)
        love.graphics.setColor(34,176,56,255)
        love.graphics.print(_text,_xeol+_weol/2,_yeol+_heol/2,0,1,1,_font:getWidth(_text)/2,_font:getHeight(_text)/2)
    end
end
